"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _getPrototypeOf = require("babel-runtime/core-js/object/get-prototype-of");

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require("babel-runtime/helpers/createClass");

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _get2 = require("babel-runtime/helpers/get");

var _get3 = _interopRequireDefault(_get2);

var _inherits2 = require("babel-runtime/helpers/inherits");

var _inherits3 = _interopRequireDefault(_inherits2);

exports.default = BaseComponent;

var _deepEqual = require("deep-equal");

var _deepEqual2 = _interopRequireDefault(_deepEqual);

var _utils = require("../utils");

var _Context = require("../Context");

var _Context2 = _interopRequireDefault(_Context);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A base form component that unifies state management and some optimization.
 * Handles also loading settings & triggering settings change.
 */
function BaseComponent(ComposedComponent) {
	var _class, _temp;

	return _temp = _class = function (_ComposedComponent) {
		(0, _inherits3.default)(BaseComponent, _ComposedComponent);

		function BaseComponent(props) {
			(0, _classCallCheck3.default)(this, BaseComponent);

			var _this = (0, _possibleConstructorReturn3.default)(this, (BaseComponent.__proto__ || (0, _getPrototypeOf2.default)(BaseComponent)).call(this, props));

			_this.onChange = _this.onChange.bind(_this);
			if (props.uiSchema && props.uiSchema["ui:settings"]) _this.loadContextSettings(props, _this.getContext());
			if (!_this.state && _this.getStateFromProps) _this.state = _this.getStateFromProps(props);
			if (props.uiSchema && props.uiSchema["ui:settings"]) _this.state = _this.loadStateSettings(props, _this.state);
			return _this;
		}

		(0, _createClass3.default)(BaseComponent, [{
			key: "loadSettings",
			value: function loadSettings(props, target, rule) {
				var _this2 = this;

				var uiSchema = props.uiSchema,
				    formContext = props.formContext;


				if (uiSchema && uiSchema["ui:settings"]) {
					var settings = formContext.settings || {};
					uiSchema["ui:settings"].forEach(function (key) {
						if (_this2.getSettingsKey(props, key) in settings) {
							if (key.match(rule)) {
								var last = key.match(/.*\/(.*)/, "$1")[1];
								var withoutLastMatch = key.match(/^(%[^/]+)?(\/.*)\/.*$/);
								var withoutLast = withoutLastMatch ? withoutLastMatch[2] : "/";
								var lastContainer = (0, _utils.parseJSONPointer)(target, withoutLast || "/", "createParents");
								lastContainer[last] = JSON.parse((0, _stringify2.default)(settings[_this2.getSettingsKey(props, key)]));
							}
						}
					});
				}
				return target;
			}
		}, {
			key: "loadContextSettings",
			value: function loadContextSettings(props, context) {
				return this.loadSettings(props, context, /^%/);
			}
		}, {
			key: "loadStateSettings",
			value: function loadStateSettings(props) {
				var state = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

				return this.loadSettings(props, state, /^((?!%))/);
			}

			// Settings are hashed with setting id, field id, field name and the options path. Settings id is optional, all the others are automatically read.

		}, {
			key: "getSettingsKey",
			value: function getSettingsKey(props, key) {
				var uiSchema = props.uiSchema,
				    id = props.idSchema.$id;

				var settingsId = uiSchema && uiSchema["ui:settingsId"] !== undefined ? uiSchema["ui:settingsId"] : "";

				if (key.match(/^%/)) {
					return key;
				} else {
					return settingsId + "#" + id + "$" + uiSchema["ui:field"] + key;
				}
			}
		}, {
			key: "componentWillReceiveProps",
			value: function componentWillReceiveProps(props) {
				if ((0, _get3.default)(BaseComponent.prototype.__proto__ || (0, _getPrototypeOf2.default)(BaseComponent.prototype), "componentWillReceiveProps", this)) {
					(0, _get3.default)(BaseComponent.prototype.__proto__ || (0, _getPrototypeOf2.default)(BaseComponent.prototype), "componentWillReceiveProps", this).call(this, props);
				} else if (this.getStateFromProps) {
					var state = this.getStateFromProps(props);
					if (state) this.setState(state);
				}
				this.updateSettingSaver(props);
			}

			// JSON Parsing will throw exceptions for paths that aren't initialized. This is intentional -
			// we don't want to save undefined values for settings that are not set.

		}, {
			key: "updateSettingSaver",
			value: function updateSettingSaver(props) {
				var _this3 = this;

				function parseSettingSaver(target, key) {
					var splits = key.split("/");
					var last = splits.pop();

					var lastContainer = (0, _utils.parseJSONPointer)(target, splits.join("/"));

					if (lastContainer && last in lastContainer) {
						return lastContainer[last];
					} else {
						throw new Error("Setting saver parsing failed");
					}
				}

				if (props.uiSchema) (props.uiSchema["ui:settings"] || []).forEach(function (key) {
					_this3.getContext().addSettingSaver(_this3.getSettingsKey(props, key), function () {
						if (key.match(/^%/)) {
							return parseSettingSaver(_this3.getContext(), key.replace(/^%[^/]*/, ""));
						} else {
							return parseSettingSaver(_this3.state, key);
						}
					});
				});
			}
		}, {
			key: "componentDidUpdate",
			value: function componentDidUpdate(prevProps, prevState) {
				var _this4 = this;

				if ((0, _get3.default)(BaseComponent.prototype.__proto__ || (0, _getPrototypeOf2.default)(BaseComponent.prototype), "componentDidUpdate", this)) {
					(0, _get3.default)(BaseComponent.prototype.__proto__ || (0, _getPrototypeOf2.default)(BaseComponent.prototype), "componentDidUpdate", this).call(this, prevProps, prevState);
				}

				if (this.props.uiSchema && (this.props.uiSchema["ui:settings"] || []).some(function (key) {
					if (key.match(/^%/)) key = key.match(/^%([^/]*)/)[1];
					if (!_deepEqual2.default.apply(undefined, (0, _toConsumableArray3.default)([prevState, _this4.state].map(function (state) {
						return (0, _utils.parseJSONPointer)(state, key, !!"safely");
					})))) {
						return true;
					}
				})) {
					this.getContext().onSettingsChange();
				}
			}
		}, {
			key: "componentDidMount",
			value: function componentDidMount() {
				this.updateSettingSaver(this.props);
				if ((0, _get3.default)(BaseComponent.prototype.__proto__ || (0, _getPrototypeOf2.default)(BaseComponent.prototype), "componentDidMount", this)) (0, _get3.default)(BaseComponent.prototype.__proto__ || (0, _getPrototypeOf2.default)(BaseComponent.prototype), "componentDidMount", this).call(this);
			}
		}, {
			key: "componentWillUnmount",
			value: function componentWillUnmount() {
				var _this5 = this;

				if (this.props.uiSchema) (this.props.uiSchema["ui:settings"] || []).forEach(function (key) {
					_this5.getContext().removeSettingSaver(_this5.getSettingsKey(_this5.props, key));
				});
				if ((0, _get3.default)(BaseComponent.prototype.__proto__ || (0, _getPrototypeOf2.default)(BaseComponent.prototype), "componentWillUnmount", this)) (0, _get3.default)(BaseComponent.prototype.__proto__ || (0, _getPrototypeOf2.default)(BaseComponent.prototype), "componentWillUnmount", this).call(this);
			}
		}, {
			key: "onChange",
			value: function onChange(formData) {
				(0, _get3.default)(BaseComponent.prototype.__proto__ || (0, _getPrototypeOf2.default)(BaseComponent.prototype), "onChange", this) ? (0, _get3.default)(BaseComponent.prototype.__proto__ || (0, _getPrototypeOf2.default)(BaseComponent.prototype), "onChange", this).call(this, formData) : this.props.onChange(formData);
			}
		}, {
			key: "getContext",
			value: function getContext() {
				return new _Context2.default(this.props.formContext.contextId);
			}
		}, {
			key: "getUUID",
			value: function getUUID() {
				return (0, _utils.getUUID)(this.props.formData) || this.props.formContext._parentLajiFormId || "root";
			}
		}, {
			key: "addSubmitHook",
			value: function addSubmitHook(hook) {
				var id = this.getUUID(this.props.formData) || this.props.formContext._parentLajiFormId || "root";
				var lajiFormInstance = new _Context2.default(this.props.formContext.contextId).formInstance;
				var relativePointer = (0, _utils.getRelativePointer)(lajiFormInstance.tmpIdTree, lajiFormInstance.state.formData, this.props.idSchema.$id, id);
				return new _Context2.default(this.props.formContext.contextId).addSubmitHook(id, relativePointer, hook);
			}
		}]);
		return BaseComponent;
	}(ComposedComponent), _class.displayName = (0, _utils.getReactComponentName)(ComposedComponent), _temp;
}