"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = undefined;

var _defineProperty2 = require("babel-runtime/helpers/defineProperty");

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

var _extends3 = require("babel-runtime/helpers/extends");

var _extends4 = _interopRequireDefault(_extends3);

var _getPrototypeOf = require("babel-runtime/core-js/object/get-prototype-of");

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require("babel-runtime/helpers/createClass");

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require("babel-runtime/helpers/inherits");

var _inherits3 = _interopRequireDefault(_inherits2);

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _class;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _utils = require("../../utils");

var _BaseComponent = require("../BaseComponent");

var _BaseComponent2 = _interopRequireDefault(_BaseComponent);

var _utils2 = require("@rjsf/core/dist/cjs/utils");

var _Context = require("../../Context");

var _Context2 = _interopRequireDefault(_Context);

var _deepmerge = require("deepmerge");

var _deepmerge2 = _interopRequireDefault(_deepmerge);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var suggestionParsers = {
	taxonGroup: function taxonGroup(suggestion) {
		return suggestion.payload ? suggestion.payload.informalTaxonGroups.map(function (item) {
			return typeof item === "string" ? item : item.id;
		}) : [];
	}
};

var parseQuery = function parseQuery(query, props, taxonGroups) {
	return (0, _keys2.default)(query).reduce(function (_query, key) {
		if ((0, _typeof3.default)(query[key]) === "object") {
			var _query$key = query[key],
			    parser = _query$key.parser,
			    field = _query$key.field;
			var _props$formData = props.formData,
			    formData = _props$formData === undefined ? {} : _props$formData;

			if (parser === "arrayJoin") {
				_query[key] = ((key === "informalTaxonGroup" ? taxonGroups : formData[field]) || []).join(",");
			}
		} else {
			_query[key] = query[key];
		}
		return _query;
	}, {});
};

/**
 * Uses AutosuggestWidget to apply autosuggested values to multiple object's fields. Options are passed to AutosuggestWidget.
 *
 * uischema = {"ui:options": {
 *  autosuggestField: <string> (field name which is used for api call. The suggestions renderer method is also defined by autosuggestField)
 *  suggestionInputField: <fieldName> (the field which is rendered as the autosuggest input)
 *  suggestionValueField: <fieldName> (the field which the value for autosuggest is pulled from)
 *  suggestionReceivers: {
 *    <fieldName>: <suggestion path>,     (when an autosuggestion is selected, these fields receive the autosuggestions value defined by suggestion path.
 *    <fieldName2>: <suggestion path 2>,   Example: autosuggestion = {key: "MLV.2", value: "kalalokki", payload: {informalGroups: ["linnut"]}}
 *   }                                              suggestionReceivers: {someFieldName: "key", someFieldName2: "/payload/informalgroups/0"}
 *                                         If fieldName start  with '$', then a function from autosuggestFieldSettings parses the suggestion. Example: $taxonGroup
 *                                         If fieldName start  with '/', it is handled as a JSON pointer.
 *  uiSchema: <uiSchema> (uiSchema which is passed to inner SchemaField)
 * }
 */

var AutosuggestField = (0, _BaseComponent2.default)(_class = function (_Component) {
	(0, _inherits3.default)(AutosuggestField, _Component);

	function AutosuggestField() {
		var _ref;

		var _temp, _this, _ret;

		(0, _classCallCheck3.default)(this, AutosuggestField);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = (0, _possibleConstructorReturn3.default)(this, (_ref = AutosuggestField.__proto__ || (0, _getPrototypeOf2.default)(AutosuggestField)).call.apply(_ref, [this].concat(args))), _this), _this.componentWillReceiveProps = function (props) {
			_this.setState(_this.getStateFromProps(props));
			if (_this.onNextTick) {
				_this.onNextTick();
				_this.onNextTick = undefined;
			}
		}, _this.getTogglePersistenceContextKey = function (props) {
			return "AUTOSUGGEST_FIELD_TOGGLE_PERSISTENCE_" + props.uiSchema["ui:options"].togglePersistenceKey;
		}, _this.getInformalTaxonGroupsPersistenceContextKey = function (props) {
			return "AUTOSUGGEST_FIELD_PERSISTENCE_" + props.uiSchema["ui:options"].informalTaxonGroupPersistenceKey;
		}, _this.getStateFromProps = function (props, toggled) {
			var schema = props.schema,
			    uiSchema = props.uiSchema,
			    formData = props.formData;

			var context = new _Context2.default(_this.props.formContext.contextId);
			var uiOptions = (0, _utils.getUiOptions)(uiSchema);
			var _uiOptions$informalTa = uiOptions.informalTaxonGroups,
			    informalTaxonGroups = _uiOptions$informalTa === undefined ? "informalTaxonGroups" : _uiOptions$informalTa,
			    informalTaxonGroupPersistenceKey = uiOptions.informalTaxonGroupPersistenceKey,
			    togglePersistenceKey = uiOptions.togglePersistenceKey;


			if (togglePersistenceKey) {
				toggled = context[_this.getTogglePersistenceContextKey(props)];
			}

			toggled = toggled !== undefined ? toggled : _this.state ? _this.state.toggled : togglePersistenceKey ? context[_this.getTogglePersistenceContextKey(props)] : false;

			var taxonGroupID = !informalTaxonGroups ? undefined : informalTaxonGroupPersistenceKey !== undefined ? context[_this.getInformalTaxonGroupsPersistenceContextKey(props)] : formData[informalTaxonGroups] ? formData[informalTaxonGroups][0] : undefined;

			var options = (0, _extends4.default)({}, uiOptions, {
				onSuggestionSelected: _this.onSuggestionSelected,
				onConfirmUnsuggested: _this.onConfirmUnsuggested,
				onInputChange: _this.onInputChange,
				isValueSuggested: _this.isValueSuggested,
				getSuggestionFromValue: _this.getSuggestionFromValue,
				onInformalTaxonGroupSelected: informalTaxonGroups ? _this.onInformalTaxonGroupSelected : undefined,
				informalTaxonGroupsValue: props.formData[informalTaxonGroups],
				taxonGroupID: taxonGroupID,
				placeholder: toggled ? _this.props.formContext.translations.UnitAutosuggestFieldTogglePlaceholder : uiOptions["ui:placeholder"]
			});

			if (uiOptions.toggleable) {
				options.toggled = toggled;
				options.onToggle = _this.onToggleChange;

				options = _this.getActiveOptions(options, toggled);
			}

			var _options = options,
			    suggestionInputField = _options.suggestionInputField;


			if (suggestionInputField && props.formData && !(0, _utils.isEmptyString)((0, _utils.parseJSONPointer)(props.formData, suggestionInputField, !!"safe"))) {
				options.value = (0, _utils.parseJSONPointer)(props.formData, suggestionInputField);
			}

			if (options.query) {
				options.query = parseQuery(options.query, props, [options.taxonGroupID]);
			}

			var innerUiSchema = (0, _utils.getInnerUiSchema)(uiSchema);
			var _uiSchemaJSONPointer = (0, _utils.uiSchemaJSONPointer)(schema, suggestionInputField);
			var suggestionInputFieldExistingUiSchema = (0, _utils.parseJSONPointer)(innerUiSchema, _uiSchemaJSONPointer);
			var _uiSchema = (0, _utils.updateSafelyWithJSONPointer)(innerUiSchema, (0, _extends4.default)({
				"ui:widget": "AutosuggestWidget"
			}, suggestionInputFieldExistingUiSchema || {}, {
				"ui:options": (0, _extends4.default)({}, (0, _utils.getUiOptions)((suggestionInputFieldExistingUiSchema || {})[suggestionInputField]), options)
			}), _uiSchemaJSONPointer);

			return { schema: schema, uiSchema: _uiSchema, toggled: toggled, taxonGroupID: taxonGroupID };
		}, _this.getActiveOptions = function (options, toggled) {
			toggled = toggled !== undefined ? toggled : (_this.state || {}).toggled;
			return toggled ? (0, _deepmerge2.default)(options, options.toggleable) : options;
		}, _this.onSuggestionSelected = function (suggestion, mounted) {
			if (suggestion === null) suggestion = undefined;

			var _this$props = _this.props,
			    formData = _this$props.formData,
			    uiSchema = _this$props.uiSchema,
			    formContext = _this$props.formContext;

			var _this$getActiveOption = _this.getActiveOptions((0, _utils.getUiOptions)(uiSchema)),
			    suggestionReceivers = _this$getActiveOption.suggestionReceivers,
			    autosuggestField = _this$getActiveOption.autosuggestField,
			    suggestionValueField = _this$getActiveOption.suggestionValueField,
			    autocopy = _this$getActiveOption.autocopy,
			    suggestionInputField = _this$getActiveOption.suggestionInputField;

			var handleSuggestionReceivers = function handleSuggestionReceivers(formData, suggestion) {
				for (var fieldName in suggestionReceivers) {
					// undefined suggestion clears value.
					var fieldVal = undefined;
					if ((typeof suggestion === "undefined" ? "undefined" : (0, _typeof3.default)(suggestion)) === "object") {
						var suggestionValPath = suggestionReceivers[fieldName];
						if (suggestionValPath[0] === "$") {
							fieldVal = suggestionParsers[suggestionValPath.substring(1)](suggestion);
						} else if (suggestionValPath[0] === "/") {
							fieldVal = (0, _utils.parseJSONPointer)(suggestion, suggestionValPath);
						} else {
							fieldVal = suggestion[suggestionValPath];
						}
					}
					formData = (0, _utils.updateFormDataWithJSONPointer)((0, _extends4.default)({}, _this.props, { formData: formData }), fieldVal, fieldName);
				}
				return formData;
			};

			if (autosuggestField === "unit") {
				var unit = suggestion.payload.unit;

				if (unit.unitType) {
					unit.informalTaxonGroups = unit.unitType;
					delete unit.unitType;
				}
				unit = (mounted && formContext.formDataTransformers || []).reduce(function (unit, _ref2) {
					var uiField = _ref2["ui:field"],
					    fieldProps = _ref2.props;

					var _ref3 = new fieldProps.registry.fields[uiField]((0, _extends4.default)({}, fieldProps, { formData: unit })),
					    _ref3$state = _ref3.state,
					    state = _ref3$state === undefined ? {} : _ref3$state;

					return state.formData;
				}, unit);
				formData = handleSuggestionReceivers(formData, {});
				formData = (0, _extends4.default)({}, (0, _utils.updateFormDataWithJSONPointer)((0, _extends4.default)({}, _this.props, { formData: formData }), undefined, suggestionValueField), unit);
				if ((0, _utils.isEmptyString)((0, _utils.parseJSONPointer)(_this.props.formData, suggestionInputField, !!"safe")) && autocopy && !(0, _utils.formDataEquals)(_this.props.formData, formData, _this.props.formContext, _this.props.idSchema.$id)) {
					_this.onNextTick = function () {
						return new _Context2.default(_this.props.formContext.contextId).sendCustomEvent(_this.props.idSchema.$id, "copy", autocopy);
					};
				}
			} else {
				formData = handleSuggestionReceivers(formData, suggestion);
			}
			if (mounted) {
				_this.props.onChange(formData);
			} else {
				if (formContext.formDataTransformers) {
					formData = formContext.formDataTransformers.reduce(function (unit, _ref4) {
						var uiField = _ref4["ui:field"],
						    fieldProps = _ref4.props;

						var changed = void 0;
						var getChanged = function getChanged(_changed) {
							changed = _changed;
						};

						var field = new fieldProps.registry.fields[uiField]((0, _extends4.default)({}, fieldProps, { formData: formData, onChange: getChanged }));
						field.onChange(field.state && field.state.formData ? field.state.formData : formData);
						return changed;
					}, formData);
				}
				var lajiFormInstance = new _Context2.default(formContext.contextId).formInstance;
				var pointer = (0, _utils.getJSONPointerFromLajiFormIdAndFormDataAndIdSchemaId)(lajiFormInstance.tmpIdTree, lajiFormInstance.state.formData, _this.props.idSchema.$id, _this.getUUID());
				var newFormData = (0, _extends4.default)({}, (0, _utils.parseJSONPointer)(lajiFormInstance.state.formData, pointer), formData);
				lajiFormInstance.onChange({ formData: (0, _utils.updateSafelyWithJSONPointer)(lajiFormInstance.state.formData, newFormData, pointer) });
			}
		}, _this.onConfirmUnsuggested = function (value) {
			var _this$props2 = _this.props,
			    formData = _this$props2.formData,
			    uiSchema = _this$props2.uiSchema;

			var _this$getActiveOption2 = _this.getActiveOptions((0, _utils.getUiOptions)(uiSchema)),
			    suggestionReceivers = _this$getActiveOption2.suggestionReceivers,
			    suggestionInputField = _this$getActiveOption2.suggestionInputField;

			(0, _keys2.default)(suggestionReceivers).forEach(function (fieldName) {
				var defaultValue = (0, _utils2.getDefaultFormState)((0, _utils.parseJSONPointer)(_this.props.schema, (0, _utils.schemaJSONPointer)(_this.props.schema, fieldName), undefined, _this.props.registry.definitions));
				formData = (0, _utils.updateFormDataWithJSONPointer)((0, _extends4.default)({}, _this.props, { formData: formData }), defaultValue, fieldName);
			});
			formData = (0, _utils.updateFormDataWithJSONPointer)((0, _extends4.default)({}, _this.props, { formData: formData }), value, suggestionInputField);
			_this.props.onChange(formData);
		}, _this.onInputChange = function (value) {
			var _this$props3 = _this.props,
			    formData = _this$props3.formData,
			    uiSchema = _this$props3.uiSchema;

			var _this$getActiveOption3 = _this.getActiveOptions((0, _utils.getUiOptions)(uiSchema)),
			    inputTransformer = _this$getActiveOption3.inputTransformer;

			if (inputTransformer) {
				var regexp = new RegExp(inputTransformer.regexp);
				if (value.match(regexp)) {
					if (!formData) formData = {};
					var formDataChange = {};
					value = value.replace(regexp, "\$1");
					if (inputTransformer.transformations) for (var transformField in inputTransformer.transformations) {
						formDataChange[transformField] = inputTransformer.transformations[transformField];
					}
					formData = (0, _extends4.default)({}, formData, formDataChange);
					_this.props.onChange(formData);
				}
			}
			return value;
		}, _this.isValueSuggested = function () {
			var _this$props4 = _this.props,
			    formData = _this$props4.formData,
			    uiSchema = _this$props4.uiSchema;

			for (var fieldName in _this.getActiveOptions((0, _utils.getUiOptions)(uiSchema)).suggestionReceivers) {
				if (!formData || !(0, _utils.parseJSONPointer)(formData, fieldName, !!"safe")) {
					return false;
				}
			}
			return true;
		}, _this.getSuggestionFromValue = function () {
			var _this$props5 = _this.props,
			    formData = _this$props5.formData,
			    uiSchema = _this$props5.uiSchema;

			var _this$getActiveOption4 = _this.getActiveOptions((0, _utils.getUiOptions)(uiSchema)),
			    suggestionValueField = _this$getActiveOption4.suggestionValueField,
			    suggestionInputField = _this$getActiveOption4.suggestionInputField;

			var suggestionValue = (0, _utils.parseJSONPointer)(formData, suggestionValueField, !!"safe");
			var suggestionInputValue = (0, _utils.parseJSONPointer)(formData, suggestionInputField, !!"safe");

			var value = suggestionInputField && formData && !(0, _utils.isEmptyString)(suggestionValue) ? suggestionInputValue : undefined;
			var key = suggestionValueField && formData && !(0, _utils.isEmptyString)(suggestionValue) ? suggestionValue : undefined;

			var suggestion = undefined;
			if (value !== undefined && key !== undefined) {
				suggestion = { value: value, key: key };
			}

			return suggestion ? _promise2.default.resolve(suggestion) : _promise2.default.reject();
		}, _this.onInformalTaxonGroupSelected = function (informalTaxonID) {
			var uiSchema = _this.props.uiSchema;

			var _this$getActiveOption5 = _this.getActiveOptions((0, _utils.getUiOptions)(uiSchema)),
			    informalTaxonGroups = _this$getActiveOption5.informalTaxonGroups,
			    informalTaxonGroupPersistenceKey = _this$getActiveOption5.informalTaxonGroupPersistenceKey;

			if (informalTaxonGroupPersistenceKey !== undefined) {
				new _Context2.default(_this.props.formContext.contextId)[_this.getInformalTaxonGroupsPersistenceContextKey(_this.props)] = informalTaxonID;
				_this.setState(_this.getStateFromProps(_this.props));
			} else {
				_this.props.onChange((0, _extends4.default)({}, _this.props.formData, (0, _defineProperty3.default)({}, informalTaxonGroups, [informalTaxonID])));
			}
		}, _this.onToggleChange = function (value) {
			var _getUiOptions = (0, _utils.getUiOptions)(_this.props.uiSchema),
			    togglePersistenceKey = _getUiOptions.togglePersistenceKey;

			if (togglePersistenceKey) {
				new _Context2.default(_this.props.formContext.contextId)[_this.getTogglePersistenceContextKey(_this.props)] = value;
			}
			_this.setState(_this.getStateFromProps(_this.props, value));
		}, _temp), (0, _possibleConstructorReturn3.default)(_this, _ret);
	}

	(0, _createClass3.default)(AutosuggestField, [{
		key: "render",
		value: function render() {
			var SchemaField = this.props.registry.fields.SchemaField;

			return _react2.default.createElement(SchemaField, (0, _extends4.default)({}, this.props, { uiSchema: this.state.uiSchema }));
		}
	}], [{
		key: "getName",
		value: function getName() {
			return "AutosuggestField";
		}
	}]);
	return AutosuggestField;
}(_react.Component)) || _class;

AutosuggestField.propTypes = {
	uiSchema: _propTypes2.default.shape({
		"ui:options": _propTypes2.default.shape({
			autosuggestField: _propTypes2.default.string.isRequired,
			suggestionInputField: _propTypes2.default.string.isRequired,
			allowNonsuggestedValue: _propTypes2.default.boolean,
			suggestionReceivers: _propTypes2.default.object.isRequired,
			inputTransformer: _propTypes2.default.shape({
				regexp: _propTypes2.default.string.isRequired,
				transformations: _propTypes2.default.object.isRequired
			}),
			informalTaxonGroups: _propTypes2.default.string,
			informalTaxonGroupPersistenceKey: _propTypes2.default.string
		}).isRequired,
		uiSchema: _propTypes2.default.object
	}).isRequired,
	schema: _propTypes2.default.shape({
		type: _propTypes2.default.oneOf(["object"])
	}).isRequired,
	formData: _propTypes2.default.object.isRequired
};
exports.default = AutosuggestField;