"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _getIterator2 = require("babel-runtime/core-js/get-iterator");

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

exports.initializeValidation = initializeValidation;
exports.transformErrors = transformErrors;

var _lajiValidate = require("laji-validate");

var _lajiValidate2 = _interopRequireDefault(_lajiValidate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initializeValidation(apiClient) {
	_lajiValidate2.default.extend(_lajiValidate2.default.validators.remote, {
		fetch: function fetch() {
			return apiClient.fetchRaw.apply(apiClient, arguments);
		}
	});
}

exports.default = function (_ref, data) {
	var error = _ref.errors,
	    warning = _ref.warnings;

	var promises = [];

	var getPromiseForValidator = function getPromiseForValidator(type, validator) {
		return _lajiValidate2.default.async(data, validator).then(function () {}).catch(function (res) {
			return res;
		}).then(function (err) {
			return { type: type, err: err };
		});
	};

	var _validators = { error: error, warning: warning };

	(0, _keys2.default)(_validators).forEach(function (type) {
		var validator = _validators[type];
		promises.push(getPromiseForValidator(type, validator));
	});

	return _promise2.default.all(promises).then(function (res) {
		var messages = res.reduce(function (arr, _ref2) {
			var type = _ref2.type,
			    err = _ref2.err;

			arr = arr.concat(getMessages(err, type));
			return arr;
		}, []);
		return _promise2.default.resolve(toErrorSchema(messages));
	});
};

function transformErrors(translations, errors) {
	return errors.map(function (error) {
		if (error.name === "type") {
			error.message = translations.TypeError + translations[error.params.type] + ".";
		} else if (error.name === "required") {
			error.message = translations.FieldIsRequired + ".";
		}
		return error;
	});
}

function getMessages(result, type) {
	var messages = [];
	for (var k in result) {
		for (var l in result[k]) {
			var message = type !== "validators" ? "[" + type + "]" + result[k][l] : result[k][l];
			messages.push({
				property: "instance." + k,
				message: message
			});
		}
	}
	return messages;
}

// these are taken from @rjsf/core to convert error messages to wanted form
// see https://github.com/mozilla-services/@rjsf/core/blob/master/src/validate.js
function toErrorSchema(errors) {
	// Transforms a jsonschema validation errors list:
	// [
	//   {property: "instance.level1.level2[2].level3", message: "err a"},
	//   {property: "instance.level1.level2[2].level3", message: "err b"},
	//   {property: "instance.level1.level2[4].level3", message: "err b"},
	// ]
	// Into an error tree:
	// {
	//   level1: {
	//     level2: {
	//       2: {level3: {errors: ["err a", "err b"]}},
	//       4: {level3: {errors: ["err b"]}},
	//     }
	//   }
	// };
	if (!errors.length) {
		return {};
	}
	return errors.reduce(function (errorSchema, error) {
		var property = error.property;
		var message = error.message;

		var path = errorPropertyToPath(property);
		var parent = errorSchema;
		var _iteratorNormalCompletion = true;
		var _didIteratorError = false;
		var _iteratorError = undefined;

		try {
			for (var _iterator = (0, _getIterator3.default)(path.slice(1)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
				var segment = _step.value;

				if (!(segment in parent)) {
					parent[segment] = {};
				}
				parent = parent[segment];
			}
		} catch (err) {
			_didIteratorError = true;
			_iteratorError = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion && _iterator.return) {
					_iterator.return();
				}
			} finally {
				if (_didIteratorError) {
					throw _iteratorError;
				}
			}
		}

		if (Array.isArray(parent.__errors)) {
			// We store the list of errors for this node in a property named __errors
			// to avoid name collision with a possible sub schema field named
			// "errors" (see `validate.createErrorHandler`).
			parent.__errors = parent.__errors.concat(message);
		} else {
			parent.__errors = [message];
		}
		return errorSchema;
	}, {});
}

var RE_ERROR_ARRAY_PATH = /\[\d+]/g;

function errorPropertyToPath(property) {
	// Parse array indices, eg. "instance.level1.level2[2].level3"
	// => ["instance", "level1", "level2", 2, "level3"]
	return property.split(".").reduce(function (path, node) {
		var match = node.match(RE_ERROR_ARRAY_PATH);
		if (match) {
			var nodeName = node.slice(0, node.indexOf("["));
			var indices = match.map(function (str) {
				return parseInt(str.slice(1, -1), 10);
			});
			path = path.concat(nodeName, indices);
		} else {
			path.push(node);
		}
		return path;
	}, []);
}